'
' MediaMonkey Script (MM3)
'
' NAME: RestorePlayList.vbs
'
' Original AUTHOR: LikeMM
'
' Saturday, March 12, 2011
'
' Why this script: If MM crashes or if the computer does not wake up from 
' standby/hibernation and needs to be rebooted, MediaMonkey will not remember 
' the last song it was playing. Nor will it remember the items in NowPlaying list. 
' This script along with the ResumePlay script by DiddeLeeDoo will restore 
' NowPlaying/song/track position nicely.

' Thanks to: DiddeLeeDoo, Lowlander & rovingcowboy

' How: When a song finishes playing, the script waits for 6 seconds for the NowPlaying list 
' to get updated in the window and then writes that list to registry. Also, when MM starts up, 
' the scripts writes the current state of MM as "crashed" in the registry. If MM exits 
' normally then this is set to false. On subsequent MM starts, the script looks for 
' the state of crash. If false, it toggles the crashed state to true and does nothing further. 
' If it finds the crashed state to be true, meaning MM was not given a chance to exit properly, 
' then it restores NowPlaying from the registry. It also sets the current song correctly.
' To top it, ResumePlay script will set the track position correctly :)

Sub OnStartUp
  Script.RegisterEvent SDB,"OnPlay","Event_OnPlay"
  Script.RegisterEvent SDB,"OnShutdown","NormalShutdown"
  
  With SDB.Registry
    If .OpenKey("RestorePlayList", False) Then
      If  .BoolValue("Crashed") Then
        Dim resPlayList, qryStr, Iter
        resPlayList = split(.StringValue("PlayList"))
        'Clear incorrect playlist due to crash
        SDB.Player.PlaylistClear
        For counter = UBound(resPlayList) to 0 Step -1
          qryStr ="AND Songs.ID = " & resPlayList(counter)
          Set Iter = SDB.Database.QuerySongs(qryStr)
          If NOT Iter.EOF Then
            SDB.Player.PlaylistAddTrack Iter.Item
          End If
        Next
        SDB.Player.CurrentSongIndex = .StringValue("CurrentSong")
      Else
        'set crashed as True by default
        .BoolValue("Crashed") = True
      End If
     .CloseKey
    End If
  End With
End Sub

Sub Event_OnPlay()
    Set PLAYTMR=SDB.CreateTimer(6000)
    Script.RegisterEvent PLAYTMR, "OnTimer","RegisterPlayList"
End Sub

Sub RegisterPlayList(Timer)
  Dim List, curSongID
  Set List = SDB.Player.CurrentPlaylist
  curSongIdx = SDB.Player.CurrentSongIndex
  Dim i, playListStr
  i=0 :   playListStr = "" :   playListMsg = ""
  
  While i<List.Count And Not Script.Terminate
    playListStr = playListStr & " " & List.Item(List.Count-i-1).ID
    playListMsg = playListMsg & " " & List.Item(List.Count-i-1).ID & "-" & List.Item(List.Count-i-1).Custom4 & VbCrLf
    i=i+1
  WEnd
  
  With SDB.Registry
    If .OpenKey("RestorePlayList", True) Then
      .StringValue("PlayList")=Trim(playListStr)
      .StringValue("CurrentSong")=curSongIdx
      .CloseKey
    End If
  End With
  Script.UnregisterEvents Timer ' Terminate usage of this timer 
  REM MsgBox(playListMsg)
End Sub

Sub NormalShutdown
  With SDB.Registry
      If .OpenKey("RestorePlayList", True) Then
       .BoolValue("Crashed")=False
       .CloseKey
      End If
  End With
End Sub